package admin.listener;


import admin.model.deepseek.resp.stream.ChatStreamResponse;
import admin.model.maxdb.resp.MaxdbStreamResponse;
import admin.modules.chatgpt.domain.ChatgptRoleRecords;
import admin.modules.chatgpt.service.ChatgptRoleRecordsService;
import admin.service.TextToSpeechService;
import admin.util.DateUtil;
import admin.util.ScreenEnum;
import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.Gson;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import okhttp3.Response;
import okhttp3.ResponseBody;
import okhttp3.sse.EventSource;
import okhttp3.sse.EventSourceListener;

import javax.annotation.Resource;
import javax.websocket.Session;
import java.net.SocketException;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author lj
 * @date 2025/2/24 9:26
 */
@Slf4j
public class DeepSeekAiListener extends EventSourceListener {

    StringBuffer stringBuffer = new StringBuffer();
    private Session session;
    private String sessionId;
    private String userName;
    private static TextToSpeechService textToSpeechService;
    private static ChatgptRoleRecordsService chatgptRoleRecordsService;

    @Resource
    public static  void  setTextToSpeechService(TextToSpeechService textToSpeechService){
        DeepSeekAiListener.textToSpeechService = textToSpeechService;
    }
    @Resource
    public static  void  setChatgptRoleRecordsService(ChatgptRoleRecordsService chatgptRoleRecordsService){
        DeepSeekAiListener.chatgptRoleRecordsService = chatgptRoleRecordsService;
    }

    public DeepSeekAiListener(Session session,String sessionId,String userName) {
        this.session = session;
        this.userName = userName;
        this.sessionId  = sessionId;
    }

    @Override
    public void onOpen(EventSource eventSource, Response response) {
        super.onOpen(eventSource, response);
        log.info("DeepSeek建立sse连接...onOpen");
    }


    @Override
    public void onEvent(EventSource eventSource, String id, String type, String data) {
        super.onEvent(eventSource, id, type, data);
        //回复内容添加  sessionId
        Gson gson= new Gson();
        try {
            MaxdbStreamResponse completionResponse = gson.fromJson(data, MaxdbStreamResponse.class);
            if (completionResponse.getIsEnd()) {
                log.info("DeepSeek返回数据结束了...onEvent");
                Map<String,Object> map = new HashMap<>();
                map.put("action","done");
                map.put("type","text");
                map.put("content",completionResponse.getContent());
                String msg = JSON.toJSONString(map);
                this.session.getBasicRemote().sendText(msg);
//                textToSpeechService.convertTextToSpeech(this.stringBuffer.toString());
                return;
            }
            if (completionResponse.getContent()!=null){
                System.out.print(completionResponse.getContent());
                this.stringBuffer.append(completionResponse.getContent());
                Map<String,Object> map = new HashMap<>();
                map.put("action","reply");
                map.put("type","text");
                map.put("content",completionResponse.getContent());
                String msg = JSON.toJSONString(map);
                this.session.getBasicRemote().sendText(msg);
            }
            //存储聊天记录
            ChatgptRoleRecords chatgptRoleRecords = new ChatgptRoleRecords();
            chatgptRoleRecords.setMessage(this.stringBuffer.toString());
            chatgptRoleRecords.setReceiverId(ScreenEnum.getByUserName(userName));
            chatgptRoleRecords.setSessionId(Integer.valueOf(sessionId));
            chatgptRoleRecords.setSenderId(10001);
            chatgptRoleRecords.setCreateTime(DateUtil.getNowTimestamp());
            chatgptRoleRecordsService.save(chatgptRoleRecords);
        }catch (Exception e){
            System.out.println(e.getMessage());
        }

//        ChatStreamResponse completionResponse = mapper.readValue(data, MaxdbStreamResponse.class);
//        if (completionResponse.getChoices()!=null && !completionResponse.getChoices().isEmpty()){
//            System.out.print(completionResponse.getChoices().get(0).getDelta().getContent());
//            this.stringBuffer.append(completionResponse.getChoices().get(0).getDelta().getContent());
//        }

    }

    @Override
    public void onClosed(EventSource eventSource) {
        super.onClosed(eventSource);
        log.info("DeepSeek关闭sse连接...onClosed");
    }

    @Override
    @SneakyThrows
    public void onFailure(EventSource eventSource, Throwable t, Response response) {
        super.onFailure(eventSource, t, response);
        if (Objects.isNull(response)) {
            return;
        }
        if(t instanceof SocketException){
            log.info("用户停止输出...onFailure");
        }else {
            ResponseBody body = response.body();
            if (Objects.nonNull(body)) {
                log.error("DeepSeek sse连接异常data：{}，异常：{}", body.string(), t);
            } else {
                log.error("DeepSeek sse连接异常data：{}，异常：{}", response, t);
            }
        }
        response.close();
        eventSource.cancel();
        log.info("eventSource cancel....onFailure");
    }
}
