package admin.modules.chatgpt.service;

import admin.base.CommonService;
import admin.base.PageInfo;
import admin.modules.chatgpt.domain.ChatgptRoleRecords;
import admin.modules.chatgpt.service.dto.ChatgptRoleRecordsDto;
import admin.modules.chatgpt.service.dto.ChatgptRoleRecordsQueryCriteria;
import org.springframework.data.domain.Pageable;
import org.springframework.scheduling.annotation.Async;

import java.util.List;
import java.util.Set;

/**
* @author wk
* @date 2023-11-27
*/
public interface ChatgptRoleRecordsService extends CommonService<ChatgptRoleRecords>  {

    static final String CACHE_KEY = "chatgptRoleRecords";

    /**
    * 查询数据分页
    * @param query 条件
    * @param pageable 分页参数
    * @return PageInfo<ChatgptRoleRecordsDto>
    */
    PageInfo<ChatgptRoleRecordsDto> queryAll(ChatgptRoleRecordsQueryCriteria query, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param query 条件参数
    * @return List<ChatgptRoleRecordsDto>
    */
    List<ChatgptRoleRecordsDto> queryAll(ChatgptRoleRecordsQueryCriteria query);

    ChatgptRoleRecords getById(Integer id);
    ChatgptRoleRecordsDto findById(Integer id);

    /**
     * 插入一条新数据。
     */
    @Async
    Integer insert(ChatgptRoleRecordsDto resources);
    Integer updateById(ChatgptRoleRecordsDto resources);
    Integer removeById(Integer id);
    Integer removeByIds(Set<Integer> ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    // void download(List<ChatgptRoleRecordsDto> all, HttpServletResponse response) throws IOException;

    PageInfo<ChatgptRoleRecordsDto> queryRoleSessionListPage(Pageable pageable, Long userId,
                                                             Integer roleDescId);

    /**
     * 累计对话文本字数
     * @param memberId
     * @return
     */
    Integer findByTextCount(Long memberId);

    /**
     * 累计会话次数
     * @param memberId
     * @return
     */
    Integer findBySessionCount(Long memberId);

    /**
     * 累计对话次数
     * @param memberId
     * @return
     */
    Integer findByDialogueCount(Long memberId);

    /**
     * 获取最新的会话id
     * @return
     */
    String  queryMaxSessionId();

    List<ChatgptRoleRecordsDto> queryTenRecords(Integer userId);
}
