package admin.modules.chatgpt.service.impl;

import admin.base.PageInfo;
import admin.base.QueryHelpMybatisPlus;
import admin.base.impl.CommonServiceImpl;
import admin.modules.chatgpt.domain.ChatgptRoleRecords;
import admin.modules.chatgpt.service.ChatgptRoleRecordsService;
import admin.modules.chatgpt.service.dto.ChatgptRoleRecordsDto;
import admin.modules.chatgpt.service.dto.ChatgptRoleRecordsQueryCriteria;
import admin.modules.chatgpt.service.mapper.ChatgptRoleRecordsMapper;

import admin.util.ConvertUtil;
import admin.util.PageUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
* @author wk
* @date 2023-11-27
*/
@Service
@AllArgsConstructor
// @CacheConfig(cacheNames = ChatgptRoleRecordsService.CACHE_KEY)
@Transactional(propagation = Propagation.SUPPORTS, readOnly = true, rollbackFor = Exception.class)
public class ChatgptRoleRecordsServiceImpl extends CommonServiceImpl<ChatgptRoleRecordsMapper, ChatgptRoleRecords> implements ChatgptRoleRecordsService {

    // private final RedisUtils redisUtils;
    private final ChatgptRoleRecordsMapper chatgptRoleRecordsMapper;

    @Override
    public PageInfo<ChatgptRoleRecordsDto> queryAll(ChatgptRoleRecordsQueryCriteria query, Pageable pageable) {
        IPage<ChatgptRoleRecords> queryPage = PageUtil.toMybatisPage(pageable);
        IPage<ChatgptRoleRecords> page = chatgptRoleRecordsMapper.selectPage(queryPage, QueryHelpMybatisPlus.getPredicate(query));
        return ConvertUtil.convertPage(page, ChatgptRoleRecordsDto.class);
    }

    @Override
    public List<ChatgptRoleRecordsDto> queryAll(ChatgptRoleRecordsQueryCriteria query){
        return ConvertUtil.convertList(chatgptRoleRecordsMapper.selectList(QueryHelpMybatisPlus.getPredicate(query)), ChatgptRoleRecordsDto.class);
    }

    @Override
    public ChatgptRoleRecords getById(Integer id) {
        return chatgptRoleRecordsMapper.selectById(id);
    }

    @Override
    // @Cacheable(key = "'id:' + #p0")
    public ChatgptRoleRecordsDto findById(Integer id) {
        return ConvertUtil.convert(getById(id), ChatgptRoleRecordsDto.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insert(ChatgptRoleRecordsDto resources) {
        ChatgptRoleRecords entity = ConvertUtil.convert(resources, ChatgptRoleRecords.class);
        return chatgptRoleRecordsMapper.insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateById(ChatgptRoleRecordsDto resources){
        ChatgptRoleRecords entity = ConvertUtil.convert(resources, ChatgptRoleRecords.class);
        int ret = chatgptRoleRecordsMapper.updateById(entity);
        // delCaches(resources.id);
        return ret;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer removeByIds(Set<Integer> ids){
        // delCaches(ids);
        return chatgptRoleRecordsMapper.deleteBatchIds(ids);
    }

    @Override
    public PageInfo<ChatgptRoleRecordsDto> queryRoleSessionListPage(Pageable pageable, Long userId,
                                                                    Integer roleDescId) {
        IPage<ChatgptRoleRecords> page = PageUtil.toMybatisPage(pageable);
        List<ChatgptRoleRecords> chatgptRoleRecords =
                chatgptRoleRecordsMapper.queryRoleSessionListPage(page, userId, roleDescId);
        page.setRecords(chatgptRoleRecords);
        return ConvertUtil.convertPage(page, ChatgptRoleRecordsDto.class);
    }

    @Override
    public Integer findByTextCount(Long memberId) {
        return this.baseMapper.findByTextCount(memberId);
    }

    @Override
    public Integer findBySessionCount(Long memberId) {
        return this.baseMapper.findBySessionCount(memberId);
    }

    @Override
    public Integer findByDialogueCount(Long memberId) {
        return this.baseMapper.findByDialogueCount(memberId);
    }

    @Override
    public String queryMaxSessionId() {
        List<ChatgptRoleRecords> list = this.baseMapper.lambdaQuery().isNotNull(ChatgptRoleRecords::getSessionId).select(ChatgptRoleRecords::getSessionId).list();
        Integer i = list.stream().map(ChatgptRoleRecords::getSessionId).max(Integer::compareTo).get();
        return String.valueOf(i+1);
    }

    @Override
    public List<ChatgptRoleRecordsDto> queryTenRecords(Integer userId) {
        List<ChatgptRoleRecordsDto> chatgptRoleRecordsDtos = chatgptRoleRecordsMapper.queryTenRecordByUserId(userId);
        return ConvertUtil.convertList(chatgptRoleRecordsDtos,ChatgptRoleRecordsDto.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer removeById(Integer id){
        Set<Integer> set = new HashSet<>(1);
        set.add(id);
        return this.removeByIds(set);
    }

    /*
    private void delCaches(Integer id) {
        redisUtils.delByKey(CACHE_KEY + "::id:", id);
    }

    private void delCaches(Set<Integer> ids) {
        for (Integer id: ids) {
            delCaches(id);
        }
    }*/

    /*
    @Override
    public void download(List<ChatgptRoleRecordsDto> all, HttpServletResponse response) throws IOException {
      List<Map<String, Object>> list = new ArrayList<>();
      for (ChatgptRoleRecordsDto chatgptRoleRecords : all) {
        Map<String,Object> map = new LinkedHashMap<>();
              map.put("发送者id", chatgptRoleRecords.getSenderId());
              map.put("接收者id", chatgptRoleRecords.getReceiverId());
              map.put("聊天消息的内容", chatgptRoleRecords.getMessage());
              map.put("创建时间", chatgptRoleRecords.getCreateTime());
        list.add(map);
      }
      FileUtil.downloadExcel(list, response);
    }*/
}
