package admin.modules.chatgpt.service.impl;

import admin.base.PageInfo;
import admin.base.QueryHelpMybatisPlus;
import admin.base.impl.CommonServiceImpl;
import admin.modules.chatgpt.domain.ChatgptRole;
import admin.modules.chatgpt.service.ChatgptRoleService;
import admin.modules.chatgpt.service.dto.ChatgptRoleDto;
import admin.modules.chatgpt.service.dto.ChatgptRoleQueryCriteria;
import admin.modules.chatgpt.service.mapper.ChatgptRoleMapper;

import admin.util.ConvertUtil;
import admin.util.PageUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
* @author wk
* @date 2023-11-27
*/
@Service
@AllArgsConstructor
// @CacheConfig(cacheNames = ChatgptRoleService.CACHE_KEY)
@Transactional(propagation = Propagation.SUPPORTS, readOnly = true, rollbackFor = Exception.class)
public class ChatgptRoleServiceImpl extends CommonServiceImpl<ChatgptRoleMapper, ChatgptRole> implements ChatgptRoleService {

    // private final RedisUtils redisUtils;
    private final ChatgptRoleMapper chatgptRoleMapper;

    @Override
    public PageInfo<ChatgptRoleDto> queryAll(ChatgptRoleQueryCriteria query, Pageable pageable) {
        IPage<ChatgptRole> queryPage = PageUtil.toMybatisPage(pageable);
        IPage<ChatgptRole> page = chatgptRoleMapper.selectPage(queryPage, QueryHelpMybatisPlus.getPredicate(query));
        return ConvertUtil.convertPage(page, ChatgptRoleDto.class);
    }

    @Override
    public List<ChatgptRoleDto> queryAll(ChatgptRoleQueryCriteria query){
        return ConvertUtil.convertList(chatgptRoleMapper.selectList(QueryHelpMybatisPlus.getPredicate(query)), ChatgptRoleDto.class);
    }

    @Override
    public ChatgptRole getById(Integer id) {
        return chatgptRoleMapper.selectById(id);
    }

    @Override
    // @Cacheable(key = "'id:' + #p0")
    public ChatgptRoleDto findById(Integer id) {
        return ConvertUtil.convert(getById(id), ChatgptRoleDto.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insert(ChatgptRoleDto resources) {
        ChatgptRole entity = ConvertUtil.convert(resources, ChatgptRole.class);
        return chatgptRoleMapper.insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateById(ChatgptRoleDto resources){
        ChatgptRole entity = ConvertUtil.convert(resources, ChatgptRole.class);
        int ret = chatgptRoleMapper.updateById(entity);
        // delCaches(resources.id);
        return ret;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int removeByIds(Set<Integer> ids){
        // delCaches(ids);
        return chatgptRoleMapper.deleteBatchIds(ids);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int removeById(Integer id){
        Set<Integer> set = new HashSet<>(1);
        set.add(id);
        return this.removeByIds(set);
    }

    /*
    private void delCaches(Integer id) {
        redisUtils.delByKey(CACHE_KEY + "::id:", id);
    }

    private void delCaches(Set<Integer> ids) {
        for (Integer id: ids) {
            delCaches(id);
        }
    }*/

    /*
    @Override
    public void download(List<ChatgptRoleDto> all, HttpServletResponse response) throws IOException {
      List<Map<String, Object>> list = new ArrayList<>();
      for (ChatgptRoleDto chatgptRole : all) {
        Map<String,Object> map = new LinkedHashMap<>();
              map.put("角色名称", chatgptRole.getRoleName());
              map.put("角色prompt", chatgptRole.getRoleMessage());
              map.put("模型", chatgptRole.getModel());
              map.put("创建时间", chatgptRole.getCreateTime());
              map.put("内容创意 该值越大，输出内容更加随机，推荐值：1", chatgptRole.getTemperature());
              map.put("最大token", chatgptRole.getMaxTokens());
              map.put("质量概率 核心采样，指选用前多少概率质量的内容，推荐值：1", chatgptRole.getTopP());
              map.put("重复词汇 输出文本同一个词汇出现情况，默认为0，值越大，重复性越小，推荐值：0", chatgptRole.getPresencePenalty());
              map.put("罕见词汇 输出文本罕见词汇出现情况，默认为0，值越大，罕见词汇越多，推荐值：0", chatgptRole.getFrequencyPenalty());
              map.put("回答数量 为每个问题回答的内容数量，当多个时，会消耗大量Token，推荐值：1", chatgptRole.getN());
        list.add(map);
      }
      FileUtil.downloadExcel(list, response);
    }*/
}
