package admin.modules.chatgpt.service.impl;

import admin.base.PageInfo;
import admin.base.QueryHelpMybatisPlus;
import admin.base.impl.CommonServiceImpl;
import admin.modules.chatgpt.domain.ChatgptRoleSession;
import admin.modules.chatgpt.service.ChatgptRoleSessionService;
import admin.modules.chatgpt.service.dto.ChatgptRoleSessionDto;
import admin.modules.chatgpt.service.dto.ChatgptRoleSessionQueryCriteria;
import admin.modules.chatgpt.service.mapper.ChatgptRoleSessionMapper;
import admin.util.ConvertUtil;
import admin.util.PageUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
* @author wk
* @date 2023-11-28
*/
@Service
@AllArgsConstructor
// @CacheConfig(cacheNames = ChatgptRoleSessionService.CACHE_KEY)
@Transactional(propagation = Propagation.SUPPORTS, readOnly = true, rollbackFor = Exception.class)
public class ChatgptRoleSessionServiceImpl extends CommonServiceImpl<ChatgptRoleSessionMapper, ChatgptRoleSession> implements ChatgptRoleSessionService {

    // private final RedisUtils redisUtils;
    private final ChatgptRoleSessionMapper chatgptRoleSessionMapper;

    @Override
    public PageInfo<ChatgptRoleSessionDto> queryAll(ChatgptRoleSessionQueryCriteria query, Pageable pageable) {
        IPage<ChatgptRoleSession> queryPage = PageUtil.toMybatisPage(pageable);
        IPage<ChatgptRoleSession> page = chatgptRoleSessionMapper.selectPage(queryPage, QueryHelpMybatisPlus.getPredicate(query));
        return ConvertUtil.convertPage(page, ChatgptRoleSessionDto.class);
    }

    @Override
    public List<ChatgptRoleSessionDto> queryAll(ChatgptRoleSessionQueryCriteria query){
        return ConvertUtil.convertList(chatgptRoleSessionMapper.selectList(QueryHelpMybatisPlus.getPredicate(query)), ChatgptRoleSessionDto.class);
    }

    @Override
    public ChatgptRoleSession getById(Integer id) {
        return chatgptRoleSessionMapper.selectById(id);
    }

    @Override
    // @Cacheable(key = "'id:' + #p0")
    public ChatgptRoleSessionDto findById(Integer id) {
        return ConvertUtil.convert(getById(id), ChatgptRoleSessionDto.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ChatgptRoleSession insert(ChatgptRoleSessionDto resources) {
        ChatgptRoleSession entity = ConvertUtil.convert(resources, ChatgptRoleSession.class);
        chatgptRoleSessionMapper.insert(entity);
        return entity;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateById(ChatgptRoleSessionDto resources){
        ChatgptRoleSession entity = ConvertUtil.convert(resources, ChatgptRoleSession.class);
        int ret = chatgptRoleSessionMapper.updateById(entity);
        // delCaches(resources.id);
        return ret;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int removeByIds(Set<Integer> ids){
        // delCaches(ids);
        return chatgptRoleSessionMapper.deleteBatchIds(ids);
    }

    @Override
    public Integer updateUpdateTime(Integer id) {
        return chatgptRoleSessionMapper.updateUpdateTime(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int removeById(Integer id){
        Set<Integer> set = new HashSet<>(1);
        set.add(id);
        return this.removeByIds(set);
    }

    /*
    private void delCaches(Integer id) {
        redisUtils.delByKey(CACHE_KEY + "::id:", id);
    }

    private void delCaches(Set<Integer> ids) {
        for (Integer id: ids) {
            delCaches(id);
        }
    }*/

    /*
    @Override
    public void download(List<ChatgptRoleSessionDto> all, HttpServletResponse response) throws IOException {
      List<Map<String, Object>> list = new ArrayList<>();
      for (ChatgptRoleSessionDto chatgptRoleSession : all) {
        Map<String,Object> map = new LinkedHashMap<>();
              map.put("名称", chatgptRoleSession.getName());
              map.put("角色描述id", chatgptRoleSession.getRoleDescId());
              map.put("用户id", chatgptRoleSession.getUserId());
              map.put("创建时间", chatgptRoleSession.getCreateTime());
        list.add(map);
      }
      FileUtil.downloadExcel(list, response);
    }*/
}
