package admin.modules.chatgpt.service.impl;

import admin.base.PageInfo;
import admin.base.QueryHelpMybatisPlus;
import admin.base.impl.CommonServiceImpl;
import admin.modules.chatgpt.domain.ChatgptRoleType;
import admin.modules.chatgpt.service.ChatgptRoleTypeService;
import admin.modules.chatgpt.service.dto.ChatgptRoleTypeDto;
import admin.modules.chatgpt.service.dto.ChatgptRoleTypeQueryCriteria;
import admin.modules.chatgpt.service.mapper.ChatgptRoleTypeMapper;
import admin.util.ConvertUtil;
import admin.util.PageUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
* @author wk
* @date 2023-11-27
*/
@Service
@AllArgsConstructor
// @CacheConfig(cacheNames = ChatgptRoleTypeService.CACHE_KEY)
@Transactional(propagation = Propagation.SUPPORTS, readOnly = true, rollbackFor = Exception.class)
public class ChatgptRoleTypeServiceImpl extends CommonServiceImpl<ChatgptRoleTypeMapper, ChatgptRoleType> implements ChatgptRoleTypeService {

    // private final RedisUtils redisUtils;
    private final ChatgptRoleTypeMapper chatgptRoleTypeMapper;

    @Override
    public PageInfo<ChatgptRoleTypeDto> queryAll(ChatgptRoleTypeQueryCriteria query, Pageable pageable) {
        IPage<ChatgptRoleType> queryPage = PageUtil.toMybatisPage(pageable);
        IPage<ChatgptRoleType> page = chatgptRoleTypeMapper.selectPage(queryPage, QueryHelpMybatisPlus.getPredicate(query));
        return ConvertUtil.convertPage(page, ChatgptRoleTypeDto.class);
    }

    @Override
    public List<ChatgptRoleTypeDto> queryAll(ChatgptRoleTypeQueryCriteria query){
        return ConvertUtil.convertList(chatgptRoleTypeMapper.selectList(QueryHelpMybatisPlus.getPredicate(query)), ChatgptRoleTypeDto.class);
    }

    @Override
    public ChatgptRoleType getById(String id) {
        return chatgptRoleTypeMapper.selectById(id);
    }

    @Override
    // @Cacheable(key = "'id:' + #p0")
    public ChatgptRoleTypeDto findById(String id) {
        return ConvertUtil.convert(getById(id), ChatgptRoleTypeDto.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insert(ChatgptRoleTypeDto resources) {
        ChatgptRoleType entity = ConvertUtil.convert(resources, ChatgptRoleType.class);
        return chatgptRoleTypeMapper.insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateById(ChatgptRoleTypeDto resources){
        ChatgptRoleType entity = ConvertUtil.convert(resources, ChatgptRoleType.class);
        int ret = chatgptRoleTypeMapper.updateById(entity);
        // delCaches(resources.id);
        return ret;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int removeByIds(Set<String> ids){
        // delCaches(ids);
        return chatgptRoleTypeMapper.deleteBatchIds(ids);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int removeById(String id){
        Set<String> set = new HashSet<>(1);
        set.add(id);
        return this.removeByIds(set);
    }

    /*
    private void delCaches(String id) {
        redisUtils.delByKey(CACHE_KEY + "::id:", id);
    }

    private void delCaches(Set<String> ids) {
        for (String id: ids) {
            delCaches(id);
        }
    }*/

    /*
    @Override
    public void download(List<ChatgptRoleTypeDto> all, HttpServletResponse response) throws IOException {
      List<Map<String, Object>> list = new ArrayList<>();
      for (ChatgptRoleTypeDto chatgptRoleType : all) {
        Map<String,Object> map = new LinkedHashMap<>();
              map.put("类别名称", chatgptRoleType.getRoleTypeName());
              map.put("排序序号", chatgptRoleType.getSortNum());
        list.add(map);
      }
      FileUtil.downloadExcel(list, response);
    }*/
}
